from fastapi import FastAPI, Depends, WebSocket, WebSocketDisconnect
from fastapi.staticfiles import StaticFiles

from .dependencies import verify_token
from .utils.email_code import smtp
from .utils.web_socket import WebSocketManager
from .routers.signin import router as signin_router
from .routers.signup import router as signup_router
from .routers.user_profile import router as user_profile_router
from .routers.user_account import router as user_account_router
from .routers.search import router as search_router
from .routers.apply import router as apply_router
from .routers.contact import router as contact_router
from .routers.group_chat import router as group_chat_router


app = FastAPI()
app.include_router(signup_router)
app.include_router(signin_router)
app.include_router(user_profile_router, dependencies=[Depends(verify_token)])
app.include_router(user_account_router, dependencies=[Depends(verify_token)])
app.include_router(search_router, dependencies=[Depends(verify_token)])
app.include_router(apply_router, dependencies=[Depends(verify_token)])
app.include_router(contact_router, dependencies=[Depends(verify_token)])
app.include_router(group_chat_router, dependencies=[Depends(verify_token)])

app.mount("/static", StaticFiles(directory="static"), name="static")


@app.on_event("shutdown")
def close_smtp():
    smtp.close()


@app.get("/")
async def main():
    return {"code": 10000, "msg": "hello world"}


ws_manager = WebSocketManager()


@app.websocket("/ws/{user_id}")
async def connect_websocket(websocket: WebSocket, user_id: str):
    await ws_manager.connect(user_id, websocket)

    try:
        while True:
            data = await websocket.receive_json()
            match data["event"]:
                case "ping":
                    await ws_manager.active_socket[user_id].send_json({"type": "pong"})
                case "friend-chat-msg":
                    if ws_manager.active_socket.get(data["receiverId"]):
                        await ws_manager.send_to_another(data["receiverId"], data)
                case "apply-friend":
                    if ws_manager.active_socket.get(data["recipient"]):
                        await ws_manager.send_to_another(data["recipient"], data)
                case "friend-added" | "friend-deleted":
                    if ws_manager.active_socket.get(data["receiverId"]):
                        await ws_manager.send_to_another(data["receiverId"], data)
                case "friend-chat-image":
                    if ws_manager.active_socket.get(data["receiverId"]):
                        await ws_manager.send_to_another(data["receiverId"], data)
                case "group-chat-creation" | "group-chat-msg" | "group-chat-image":
                    receiver_ids = data["receiverIds"]
                    del data["receiverIds"]
                    await ws_manager.broadcast(data, receiver_ids)

    except WebSocketDisconnect:
        print(f"{user_id} disconnect")
        ws_manager.disconnect(user_id)
